use crate::database::controller::connector::establish_connection;
use crate::helper::settings::Settings;
use crate::modules::api::members::member_communication::{CommunicationTarget, CommunicationType};
use crate::schema::communication_targets::dsl::{communication_targets, target_id};
use rocket::State;
use diesel::{ExpressionMethods, RunQueryDsl, QueryDsl, JoinOnDsl};

/// Updates communication target via API.
/// Will only change field if field is Some, will ignore field if it's None.
///
/// Parameters:
/// * settings: Settings as managed State
/// * com_target_id: uuid of communication target to change
/// * com_target: transmitted CommunicationTarget struct, (all fields optional)
pub fn update_communication_target(
    settings: &State<Settings>,
    com_target_id: uuid::Uuid,
    com_target: CommunicationTarget,
) -> Result<usize, diesel::result::Error> {
    let connection = establish_connection(settings);

    match diesel::update(communication_targets.filter(target_id.eq(com_target_id)))
        .set(&com_target)
        .execute(&connection)
    {
        Ok(affected_rows) => Ok(affected_rows),
        Err(e) => {
            warn!("Couldn't update communication target: {}", e);
            Err(e)
        }
    }
}

pub fn get_member_communication_types(settings: &State<Settings>) -> Result<Vec<CommunicationType>, diesel::result::Error> {
    use crate::schema::communication_types::dsl::*;

    let connection = establish_connection(settings);

    let data: Result<Vec<CommunicationType>, diesel::result::Error> = communication_types
        .select((type_id, type_name))
        .load(&connection);

    match data {
        Ok(data) => Ok(data),
        Err(e) => {
            error!("Couldn't get communication types: {}", e);
            Err(e)
        }
    }
}

/// Returns list of all email addresses belonging to the specified member
pub fn get_member_email_addresses(settings: &State<Settings>, member_id: uuid::Uuid) -> Result<Vec<String>, diesel::result::Error>{
    use crate::schema::communication_targets::dsl::*;

    let connection = establish_connection(settings);

    let email_type_id = uuid::Uuid::parse_str(&settings.mail.communication_email_type_id).expect("No valid communication_email_type_id set in config! Cant send emails!");

    let data: Result<Vec<String>, diesel::result::Error> = communication_targets.select(value).filter(com_type.eq(email_type_id)).filter(entity_id.eq(member_id)).load(&connection);
    match data{
        Ok(data) => Ok(data),
        Err(e) => {
            error!("Couldn't get email addresses for member: {}", e);
            Err(e)
        },
    }
}

pub fn get_group_email_addresses(settings: &State<Settings>, group_id2: uuid::Uuid) -> Result<Vec<String>, diesel::result::Error>{
    use crate::schema::communication_targets::dsl::*;
    use crate::schema::groups_entities::dsl::*;

    let connection = establish_connection(settings);
    let email_type_id = uuid::Uuid::parse_str(&settings.mail.communication_email_type_id).expect("No valid communication_email_type_id set in config! Cant send emails!");

    let data: Result<Vec<String>, diesel::result::Error> = communication_targets.distinct().left_join(groups_entities.on(crate::schema::groups_entities::entity_id.eq(crate::schema::communication_targets::entity_id))).filter(group_id.eq(group_id2)).select(value).filter(com_type.eq(email_type_id)).load(&connection);
    match data{
        Ok(data) => Ok(data),
        Err(e) => {
            error!("Couldn't get email addresses for group: {}", e);
            Err(e)
        },
    }
}

pub fn get_unit_email_addresses(settings: &State<Settings>,unit_id2: uuid::Uuid) -> Result<Vec<String>, diesel::result::Error>{
    use crate::schema::communication_targets::dsl::*;
    use crate::schema::units_members::dsl::*;

    let connection = establish_connection(settings);
    let email_type_id = uuid::Uuid::parse_str(&settings.mail.communication_email_type_id).expect("No valid communication_email_type_id set in config! Cant send emails!");

    let data: Result<Vec<String>, diesel::result::Error> = communication_targets.distinct().left_join(units_members.on(crate::schema::units_members::member_id.eq(crate::schema::communication_targets::entity_id))).filter(unit_id.eq(unit_id2)).select(value).filter(com_type.eq(email_type_id)).load(&connection);
    match data{
        Ok(data) => Ok(data),
        Err(e) => {
            error!("Couldn't get email addresses for unit: {}", e);
            Err(e)
        },
    }
}