use bigdecimal::BigDecimal;
use diesel::{ExpressionMethods, OptionalExtension, QueryDsl, RunQueryDsl};
use rocket::State;

use crate::database::controller::connector::establish_connection;
use crate::schema::personnel_billing;
use crate::Settings;

#[derive(Queryable, Clone, AsChangeset, Insertable)]
#[table_name = "personnel_billing"]
#[changeset_options(treat_none_as_null = "true")]
#[primary_key(position_instance_id)]
pub struct RawPersonnelBilling {
    pub(crate) position_instance_id: uuid::Uuid,
    pub(crate) member_id: uuid::Uuid,
    pub(crate) fulfilled_time: i32,
    pub(crate) money_for_time: BigDecimal,
    pub(crate) money_from_lump_sum: BigDecimal,
    pub(crate) total_money: BigDecimal,
}

pub fn remove(settings: &State<Settings>, position_instance_id: uuid::Uuid) -> Result<usize, diesel::result::Error> {
    let connection = establish_connection(settings);

    match diesel::delete(personnel_billing::table).filter(personnel_billing::dsl::position_instance_id.eq(position_instance_id)).execute(&connection) {
        Ok(num) => Ok(num),
        Err(e) => {
            error!("Couldn't remove personnel billing entry: {}", e);
            Err(e)
        }
    }
}

pub fn create(settings: &State<Settings>, data: RawPersonnelBilling) -> Result<(), diesel::result::Error> {
    //Delete existing entry if exists
    remove(settings, data.position_instance_id);

    let connection = establish_connection(settings);
    match diesel::insert_into(personnel_billing::table).values(data).execute(&connection) {
        Ok(_) => Ok(()),
        Err(e) => {
            error!("Couldn't insert personnel billing entry: {}", e);
            Err(e)
        }
    }
}

pub fn read(settings: &State<Settings>, position_instance_id: uuid::Uuid) -> Result<Option<RawPersonnelBilling>, diesel::result::Error> {
    let connection = establish_connection(settings);

    match personnel_billing::table.filter(personnel_billing::dsl::position_instance_id.eq(position_instance_id)).get_result(&connection).optional() {
        Ok(res) => Ok(res),
        Err(e) => {
            error!("Couldn't get personnel billing details: {}", e);
            Err(e)
        }
    }
}