use diesel::{ExpressionMethods, QueryDsl, RunQueryDsl, sql_query};
use diesel::pg::types::sql_types::Uuid;
use rocket::State;

use crate::database::controller::connector::establish_connection;
use crate::Settings;

#[derive(Queryable, Clone, Deserialize, Serialize)]
pub struct BillingState {
    entity_id: uuid::Uuid,
    name: String,
    description: Option<String>,
    pub(crate) final_approve: bool,
}

pub fn get_billing_states(settings: &State<Settings>) -> Result<Vec<BillingState>, diesel::result::Error> {
    use crate::schema::billing_states::dsl::*;

    let connection = establish_connection(settings);

    match billing_states.order(order.asc()).select((entity_id, name, description, final_approve)).get_results(&connection) {
        Ok(bs) => Ok(bs),
        Err(e) => {
            error!("Couldn't get billing states: {}", e);
            Err(e)
        }
    }
}

pub fn get_billing_state(settings: &State<Settings>, state: uuid::Uuid) -> Result<BillingState, diesel::result::Error> {
    use crate::schema::billing_states::dsl::*;

    let connection = establish_connection(settings);

    match billing_states.select((entity_id, name, description, final_approve)).filter(entity_id.eq(state)).get_result(&connection) {
        Ok(bs) => Ok(bs),
        Err(e) => {
            error!("Couldn't get billing state: {}", e);
            Err(e)
        }
    }
}

pub fn get_billing_states_for_event(settings: &State<Settings>, event: uuid::Uuid) -> Result<Vec<uuid::Uuid>, diesel::result::Error> {
    use crate::schema::eu_instances::dsl::*;

    let connection = establish_connection(settings);

    match eu_instances.filter(event_id.eq(event)).filter(billing_state_id.is_not_null()).select(billing_state_id).get_results(&connection) {
        Ok(states) => Ok(states.iter().map(|state: &Option<uuid::Uuid>| state.clone().unwrap()).collect()),
        Err(e) => {
            error!("Couldn't get billing states from instance for event: {}", e);
            Err(e)
        }
    }
}

#[derive(QueryableByName)]
struct TempQuery {
    #[sql_type = "Uuid"]
    pub(crate) billing_state_id: uuid::Uuid,
}

pub fn get_min_billing_states_for_event(settings: &State<Settings>, event: uuid::Uuid) -> Result<Option<uuid::Uuid>, diesel::result::Error> {
    let connection = establish_connection(settings);

    let res: Result<Option<TempQuery>, diesel::result::Error> = sql_query("SELECT billing_state_id FROM eu_instances AS eui INNER JOIN billing_states bs on eui.billing_state_id = bs.entity_id WHERE event_id = $1 ORDER BY bs.order ASC LIMIT 1;").bind::<crate::diesel::sql_types::Uuid, _>(event).get_result(&connection);
    match res {
        Ok(res) => {
            match res {
                Some(res) => Ok(Some(res.billing_state_id)),
                None => Ok(None),
            }
        },
        Err(e) => {
            error!("Couldn't get min billing states from instance for event: {}", e);
            Err(e)
        }
    }
}