use chrono::Utc;
use diesel::{ExpressionMethods, QueryDsl, RunQueryDsl, sql_query};
use diesel::sql_types::{Jsonb, Nullable, SmallInt, Text, Timestamp, Uuid};
use rocket::State;

use crate::database::controller::connector::establish_connection;
use crate::schema::eu_position_instances;
use crate::Settings;

#[derive(Queryable, Clone, Deserialize, Serialize, AsChangeset, Insertable)]
#[table_name = "eu_position_instances"]
#[changeset_options(treat_none_as_null = "true")]
#[primary_key(position_instance_id)]
pub struct RawPositionInstance {
    pub(crate) instance_id: uuid::Uuid,
    pub(crate) position_id: uuid::Uuid,
    pub(crate) taken_by: Option<uuid::Uuid>,
    pub(crate) position_instance_id: uuid::Uuid,
    pub(crate) real_start_time: Option<chrono::DateTime<Utc>>,
    pub(crate) real_end_time: Option<chrono::DateTime<Utc>>,
    pub(crate) billable: Option<bool>,
}

#[derive(Queryable, Clone, Deserialize, Serialize, AsChangeset, Insertable)]
#[table_name = "eu_position_instances"]
#[changeset_options(treat_none_as_null = "true")]
#[primary_key(position_instance_id)]
pub struct RawPositionInstanceInsert {
    pub(crate) instance_id: uuid::Uuid,
    pub(crate) position_id: uuid::Uuid,
    pub(crate) taken_by: Option<uuid::Uuid>,
    pub(crate) position_instance_id: Option<uuid::Uuid>,
    pub(crate) real_start_time: Option<chrono::DateTime<Utc>>,
    pub(crate) real_end_time: Option<chrono::DateTime<Utc>>,
}

pub fn create(settings: &State<Settings>, data: RawPositionInstanceInsert) -> Result<RawPositionInstance, diesel::result::Error> {
    let connection = establish_connection(settings);
    use crate::schema::eu_position_instances::dsl::eu_position_instances;

    match diesel::insert_into(eu_position_instances).values(data).get_result::<RawPositionInstance>(&connection) {
        Ok(res) => Ok(res),
        Err(e) => {
            error!("Couldn't insert new PositionInstance: {}", e);
            Err(e)
        }
    }
}

pub fn delete(settings: &State<Settings>, position_instance_id: uuid::Uuid) -> Result<(), diesel::result::Error> {
    let connection = establish_connection(settings);

    use crate::schema::eu_position_instances::dsl::eu_position_instances;
    match diesel::delete(eu_position_instances).filter(crate::schema::eu_position_instances::dsl::position_instance_id.eq(position_instance_id)).execute(&connection) {
        Ok(_) => Ok(()),
        Err(e) => {
            error!("Couldn't delete position_instance {}: {}", position_instance_id, e);
            Err(e)
        }
    }
}

#[derive(Queryable, Clone, Deserialize, Serialize, AsChangeset, Insertable)]
#[table_name = "eu_position_instances"]
#[primary_key(position_instance_id)]
pub struct RawPositionInstanceChangeset {
    pub(crate) instance_id: Option<uuid::Uuid>,
    pub(crate) position_id: Option<uuid::Uuid>,
    pub(crate) taken_by: Option<Option<uuid::Uuid>>,
    pub(crate) position_instance_id: uuid::Uuid,
    pub(crate) real_start_time: Option<Option<chrono::DateTime<Utc>>>,
    pub(crate) real_end_time: Option<Option<chrono::DateTime<Utc>>>,
    pub(crate) billable: Option<Option<bool>>
}

pub fn update(settings: &State<Settings>, patch: RawPositionInstanceChangeset) -> Result<RawPositionInstance, diesel::result::Error> {
    use crate::schema::eu_position_instances::dsl::*;

    let connection = establish_connection(settings);
    match diesel::update(eu_position_instances.filter(position_instance_id.eq(patch.position_instance_id))).set(patch).get_result(&connection) {
        Ok(update) => Ok(update),
        Err(e) => {
            error!("Couldn't patch position instance: {}", e);
            Err(e)
        }
    }
}

/// Replaces null values for real_start_time and real_end_time in eu_position_instances to the real times their related instance
/// *Warning: Should be run after [crate::database::controller::events::instances::instances::set_missing_real_times_to_planned_times]*
pub fn set_missing_real_times_to_instance_times(settings: &State<Settings>, event: uuid::Uuid) -> Result<(), diesel::result::Error> {
    use diesel::sql_query;

    let connection = establish_connection(settings);
    match sql_query("UPDATE eu_position_instances AS epi SET real_start_time = ei.real_start_time FROM eu_instances AS ei WHERE epi.real_start_time IS NULL AND ei.instance_id = epi.instance_id AND ei.event_id = $1;").bind::<diesel::sql_types::Uuid, _>(event).execute(&connection) {
        Ok(res) => (),
        Err(e) => {
            error!("Couldn't set missing real times to planned times for position_instances: {}", e);
            return Err(e)
        }
    }
    match sql_query("UPDATE eu_position_instances AS epi SET real_end_time = ei.real_end_time FROM eu_instances AS ei WHERE epi.real_end_time IS NULL AND ei.instance_id = epi.instance_id AND ei.event_id = $1;").bind::<diesel::sql_types::Uuid, _>(event).execute(&connection) {
        Ok(res) => Ok(()),
        Err(e) => {
            error!("Couldn't set missing real times to planned times for position_instances: {}", e);
            Err(e)
        }
    }
}