use rocket::State;
use crate::helper::settings::Settings;
use crate::database::controller::connector::establish_connection;
use diesel::{sql_query, RunQueryDsl, ExpressionMethods};
use diesel::sql_types::{Text};
use crate::database::model::login_protection::LoginAttemptsResult;




/// Checks if maximum login attempts exceeded. Locks account if exceeded
/// Returns:
/// * Ok(true) if login attempts exceeded and account got locked
/// * Ok(false) if login attempts not exceeded
/// * Err(diesel::result::Error) if database error occured
pub fn login_attempts_exceeded(settings: &State<Settings>, email: String) -> Result<bool, diesel::result::Error>{
    let connection = establish_connection(settings);

    let result : Result<LoginAttemptsResult, diesel::result::Error> = sql_query("SELECT COUNT(*) AS count, MAX(timestamp) AS last_timestamp FROM login_attempts WHERE email=$1 AND timestamp > (now() - interval '1800 seconds')").bind::<Text, _>(email.clone()).get_result(&connection);
    let result = match result{
        Ok(res) => res,
        Err(e) => {
            error!("Couldn't check for login attempts: {}", e);
            return Err(e)
        }
    };

    if result.count > settings.application.max_login_attempts as i64 {
        Ok(true)
    }else{
        Ok(false)
    }
}

/// Checks if maximum login attempts for username exceeded. Locks account if exceeded
/// Returns:
/// * Ok(true) if login attempts exceeded and account got locked
/// * Ok(false) if login attempts not exceeded
/// * Err(diesel::result::Error) if database error occured
pub fn login_attempts_usernames_exceeded(settings: &State<Settings>, username: String) -> Result<bool, diesel::result::Error>{
    let connection = establish_connection(settings);

    let result : Result<LoginAttemptsResult, diesel::result::Error> = sql_query("SELECT COUNT(*) AS count, MAX(timestamp) AS last_timestamp FROM login_attempts_usernames WHERE username=$1 AND timestamp > (now() - interval '1800 seconds')").bind::<Text, _>(username.clone()).get_result(&connection);
    let result = match result{
        Ok(res) => res,
        Err(e) => {
            error!("Couldn't check for login attempts: {}", e);
            return Err(e)
        }
    };

    if result.count > settings.application.max_login_attempts as i64 {
        Ok(true)
    }else{
        Ok(false)
    }
}

pub(crate) fn add_login_attempt(settings: &State<Settings>, email2: String) -> Result<(), diesel::result::Error>{
    use crate::schema::login_attempts::dsl::{login_attempts, email};
    let connection = establish_connection(settings);

    match diesel::insert_into(login_attempts).values(email.eq(email2)).execute(&connection){
        Ok(_) => Ok(()),
        Err(e) => {
            error!("Couldn't write login attempt into DB! {}", e);
            Err(e)
        }
    }
}

pub(crate) fn add_login_attempt_username(settings: &State<Settings>, username2: String) -> Result<(), diesel::result::Error>{
    use crate::schema::login_attempts_usernames::dsl::*;
    let connection = establish_connection(settings);

    match diesel::insert_into(login_attempts_usernames).values(username.eq(username2)).execute(&connection){
        Ok(_) => Ok(()),
        Err(e) => {
            error!("Couldn't write login attempt into DB! {}", e);
            Err(e)
        }
    }
}