use std::convert::TryInto;
use std::ops::Deref;

use rocket::serde::json::Json;
use rocket::State;

use crate::database::controller::billing::personnel_billing_rates::{get_billing_rates, RawBillingRate};
use crate::helper::session_cookies::model::SessionCookie;
use crate::helper::translate_diesel_error::translate_diesel;
use crate::modules::api::member_management::controller::parser::parse_member_cookie;
use crate::modules::api::model::api_outcome::{ApiError, ApiErrorWrapper};
use crate::Settings;

#[derive(Queryable, Clone, Serialize)]
pub struct BillingRate {
    billing_rate_id: uuid::Uuid,
    name: String,
    description: Option<String>,
    active: bool,
    payment_per_hour: String,
    lump_sum: String,
}

impl From<RawBillingRate> for BillingRate {
    fn from(data: RawBillingRate) -> Self {
        BillingRate {
            billing_rate_id: data.billing_rate_id,
            name: data.name,
            description: data.description,
            active: data.active,
            payment_per_hour: data.payment_per_hour.to_string(),
            lump_sum: data.lump_sum.to_string(),
        }
    }
}

#[get("/api/personnel_billing_rates", format = "json")]
pub fn read_personnel_billing_rates(settings: &State<Settings>, cookie: SessionCookie) -> Result<Json<Vec<BillingRate>>, Json<ApiErrorWrapper>> {
    let caller = parse_member_cookie(cookie.member)?;

    //TODO: add overwrite if member_responsible
    if !caller.has_permission(crate::permissions::modules::event_billing::VIEW.to_string()) {
        return Err(Json(ApiError::new(403, "Keine Berechtigung Abrechnungsstatusse abzurufen!".to_string()).to_wrapper()));
    }

    match get_billing_rates(settings) {
        Ok(states) => Ok(Json(states.iter().map(|mut state| state.clone().into()).collect())),
        Err(e) => Err(translate_diesel(e)),
    }
}