use std::sync::Arc;

use lettre::Message;
use rocket::serde::json::Json;
use rocket::State;

use crate::database::controller::api_communication_targets::{get_group_email_addresses, get_member_email_addresses, get_unit_email_addresses};
use crate::database::controller::groups::get_group;
use crate::database::controller::members::check_access_to_resource;
use crate::database::controller::units::get_unit;
use crate::helper::mail_queue::queue::MailQueue;
use crate::helper::session_cookies::model::SessionCookie;
use crate::helper::settings::Settings;
use crate::helper::translate_diesel_error::translate_diesel;
use crate::modules::api::member_management::controller::parser::parse_member_cookie;
use crate::modules::api::model::api_outcome::{ApiError, ApiErrorWrapper};

#[derive(Queryable, Clone, Deserialize, Serialize)]
pub struct ApiEmail {
    pub(crate) to: Option<Vec<String>>,
    pub(crate) to_members: Option<Vec<uuid::Uuid>>,
    pub(crate) cc: Option<Vec<String>>,
    pub(crate) cc_members: Option<Vec<uuid::Uuid>>,
    pub(crate) bcc: Option<Vec<String>>,
    pub(crate) bcc_members: Option<Vec<uuid::Uuid>>,
    pub(crate) reply_to: Option<String>,
    pub(crate) selected_groups: Option<Vec<uuid::Uuid>>,
    pub(crate) selected_units: Option<Vec<uuid::Uuid>>,
    pub(crate) subject: String,
    pub(crate) body: String,
}

#[post("/api/communicator/email", format = "json", data = "<mail>")]
pub fn create_email(mq: &State<Arc<MailQueue>>, settings: &State<Settings>, cookie: SessionCookie, mail: Json<ApiEmail>) -> Result<(), Json<ApiErrorWrapper>> {
    let caller = parse_member_cookie(cookie.member)?;
    if !caller.has_permission(crate::permissions::modules::communicator::email::SEND.to_string()) {
        return Err(Json(ApiError::new(403, "Keine Berechtigung Email zu versenden!".to_string()).to_wrapper()));
    }

    let maildata = mail.into_inner();

    let mut mail = Message::builder()
        .from(settings.mail.from.clone().parse().unwrap())
        .reply_to(settings.mail.reply_to.clone().parse().unwrap());

    if let Some(receivers) = maildata.to {
        for receiver in receivers {
            match receiver.parse() {
                Ok(receiver) => {
                    mail = mail.to(receiver);
                },
                Err(e) => {
                    warn!("Couldn't parse to address: {}", e);
                    return Err(Json(ApiError::new(400, "Das An Feld enthält ungültige E-Mail Adressen!".to_string()).to_wrapper()));
                }
            }
        }
    }
    if let Some(to_members) = maildata.to_members {
        for member_id in to_members {
            match get_member_email_addresses(settings, member_id) {
                Ok(mut addresses) => {
                    for address in addresses {
                        match address.parse() {
                            Ok(address) => {
                                mail = mail.to(address);
                            },
                            Err(e) => {
                                error!("Couldn't parse members email address {}: {}", address, e);
                                //Do not return error here, otherwise whole delivery would fail if one member has an falsy email address
                            }
                        }
                    }
                }
                Err(e) => {
                    return Err(translate_diesel(e))
                }
            }
        }
    }

    if let Some(ccers) = maildata.cc {
        for cc in ccers {
            match cc.parse() {
                Ok(cc) => {
                    mail = mail.cc(cc);
                },
                Err(e) => {
                    warn!("Couldn't parse cc address: {}", e);
                    return Err(Json(ApiError::new(400, "Das CC Feld enthält ungültige E-Mail Adressen!".to_string()).to_wrapper()));
                }
            }
        }
    }
    if let Some(cc_members) = maildata.cc_members {
        for member_id in cc_members {
            match get_member_email_addresses(settings, member_id) {
                Ok(mut addresses) => {
                    for address in addresses {
                        match address.parse() {
                            Ok(address) => {
                                mail = mail.cc(address);
                            },
                            Err(e) => {
                                error!("Couldn't parse members email address {}: {}", address, e);
                                //Do not return error here, otherwise whole delivery would fail if one member has an falsy email address
                            }
                        }
                    }
                }
                Err(e) => {
                    return Err(translate_diesel(e))
                }
            }
        }
    }
    if let Some(bccers) = maildata.bcc {
        for bcc in bccers {
            match bcc.parse() {
                Ok(bcc) => {
                    mail = mail.bcc(bcc);
                },
                Err(e) => {
                    warn!("Couldn't parse bcc address: {}", e);
                    return Err(Json(ApiError::new(400, "Das BCC Feld enthält ungültige E-Mail Adressen!".to_string()).to_wrapper()));
                }
            }
        }
    }
    if let Some(bcc_members) = maildata.bcc_members {
        for member_id in bcc_members {
            match get_member_email_addresses(settings, member_id) {
                Ok(mut addresses) => {
                    for address in addresses {
                        match address.parse() {
                            Ok(address) => {
                                mail = mail.bcc(address);
                            },
                            Err(e) => {
                                error!("Couldn't parse members email address {}: {}", address, e);
                                //Do not return error here, otherwise whole delivery would fail if one member has an falsy email address
                            }
                        }
                    }
                }
                Err(e) => {
                    return Err(translate_diesel(e))
                }
            }
        }
    }

    if let Some(groups) = maildata.selected_groups {
        for group in groups {
            if !check_access_to_resource(settings, caller.entity_id, group, crate::permissions::modules::communicator::email::SEND) {
                match get_group(settings, group) {
                    Ok(group) => {
                        return Err(Json(ApiError::new(403, format!("Keine Berechtigung eine Email an die Gruppe {} zu schicken!", group.name)).to_wrapper()))
                    }
                    Err(e) => {
                        return Err(translate_diesel(e))
                    }
                }
            }
            match get_group_email_addresses(settings, group) {
                Ok(mut addresses) => {
                    for address in addresses {
                        match address.parse() {
                            Ok(address) => {
                                mail = mail.bcc(address);
                            },
                            Err(e) => {
                                error!("Couldn't parse members email address {}: {}", address, e);
                                //Do not return error here, otherwise whole delivery would fail if one member has an falsy email address
                            }
                        }
                    }
                }
                Err(e) => {
                    return Err(translate_diesel(e))
                }
            }
        }
    }

    match maildata.selected_units {
        None => {}
        Some(units) => {
            for unit in units {
                if !check_access_to_resource(settings, caller.entity_id, unit, crate::permissions::modules::communicator::email::SEND) {
                    match get_unit(settings, unit) {
                        Ok(unit) => {
                            return Err(Json(ApiError::new(403, format!("Keine Berechtigung eine Email an die Einheit {} zu schicken!", unit.name)).to_wrapper()))
                        }
                        Err(e) => {
                            return Err(translate_diesel(e))
                        }
                    }
                }
                match get_unit_email_addresses(settings, unit) {
                    Ok(mut addresses) => {
                        for address in addresses {
                            match address.parse() {
                                Ok(address) => {
                                    mail = mail.bcc(address);
                                },
                                Err(e) => {
                                    error!("Couldn't parse members email address {}: {}", address, e);
                                    //Do not return error here, otherwise whole delivery would fail if one member has an falsy email address
                                }
                            }
                        }
                    }
                    Err(e) => {
                        return Err(translate_diesel(e))
                    }
                }
            }
        }
    }

    let mail = mail.body(maildata.body).unwrap();

    match mq.add_mail(mail) {
        Ok(_) => Ok(()),
        Err(_) => Err(Json(ApiError::new(500, "Couldn't add mail to mail queue!".to_string()).to_wrapper()))
    }
}
