use rocket::State;
use crate::helper::settings::Settings;
use crate::helper::session_cookies::model::SessionCookie;
use crate::modules::api::model::api_outcome::{ApiErrorWrapper, ApiError};
use rocket::serde::json::Json;
use crate::modules::api::member_management::controller::parser::{parse_member_cookie, parse_uuid_string};
use crate::database::model::organisers::Organiser;

use crate::database::controller::organisers::{get_organisers, get_organiser_count, get_organiser};
use crate::helper::translate_diesel_error::translate_diesel;

#[derive(Queryable, Clone, Deserialize, Serialize)]
pub struct OrganiserList{
    pub(crate) organisers: Vec<Organiser>,
    pub(crate) total_organiser_count: i64,
}

/// Returns detailed organiser with specified entity_id
/// Required permissions:
/// * modules.event_management.organisers.view (optional, limited set will provided if permission missing)
#[get("/api/event_organisers/<entity_id>", format = "json")]
pub fn read_organiser(
    settings: &State<Settings>,
    cookie: SessionCookie,
    entity_id: String,
) -> Result<Json<Organiser>, Json<ApiErrorWrapper>>{
    let caller = parse_member_cookie(cookie.member)?;
    let entity_id = parse_uuid_string(entity_id)?;

    match get_organiser(settings, entity_id){
        Ok(organiser) => {
            if caller.has_permission(crate::permissions::modules::event_management::organisers::VIEW.to_string()){
                Ok(Json(organiser))
            }else{
                let mut org = organiser;
                org.phone = None;
                org.email = None;
                org.other = None;
                Ok(Json(org))
            }

        },
        Err(e) => Err(translate_diesel(e))
    }
}

#[get("/api/event_organisers?<entries>&<page>&<q>", format = "json")]
pub fn read_organisers(
    settings: &State<Settings>,
    cookie: SessionCookie,
    entries: Option<i64>, //TODO: replace with limit/offset
    page: Option<i64>,
    q: Option<String>
) -> Result<Json<OrganiserList>, Json<ApiErrorWrapper>> {
    let caller = parse_member_cookie(cookie.member)?;

    if !caller.has_permission(crate::permissions::modules::event_management::organisers::VIEW.to_string()){
        return Err(Json(ApiError::new(403, "Keine Berechtigung Veranstalter abzurufen!".to_string()).to_wrapper()))
    }

    //Apply default limit of 20 entries per page if none given
    let entries = match entries{
        Some(entries) => {
            if entries > 0{
                entries
            }else{
                20
            }
        }
        None => 20
    };

    //Apply default page 1 if no page given
    let page = match page{
        Some(page) => {
            if page > 0{
                page
            }else{
                1
            }
        }
        None => 1
    };

    let mut res = OrganiserList{
        organisers: vec![],
        total_organiser_count: 0
    };

    match get_organisers(settings, entries, (page-1)*entries, q){
        Ok(organisers) => res.organisers = organisers,
        Err(e) => return Err(translate_diesel(e))
    };

    match get_organiser_count(settings){
        Ok(count) => res.total_organiser_count = count,
        Err(e) => return Err(translate_diesel(e))
    };

    Ok(Json(res))
}