use chrono::NaiveDateTime;
use rocket::serde::json::Json;
use rocket::State;

use crate::database::controller::entities::generate_entity;
use crate::database::controller::events::add_event;
use crate::database::model::events::Event;
use crate::helper::session_cookies::model::SessionCookie;
use crate::helper::settings::Settings;
use crate::helper::translate_diesel_error::translate_diesel;
use crate::modules::api::member_management::controller::parser::{parse_member_cookie, parse_option_uuid};
use crate::modules::api::model::api_outcome::{ApiError, ApiErrorWrapper};

#[derive(Queryable, Clone, Deserialize, Serialize)]
pub struct CreateEventData {
    pub(crate) name: String,
    pub(crate) start: String,
    pub(crate) end: String,
    pub(crate) site: Option<String>,
    pub(crate) organiser_id: Option<String>,
    pub(crate) etype: Option<String>,
    pub(crate) contact_on_site_name: Option<String>,
    pub(crate) contact_on_site_phone: Option<String>,
    pub(crate) member_responsible: Option<String>,
    pub(crate) related_group: Option<String>,
    pub(crate) other: Option<String>,
    pub(crate) other_intern: Option<String>,
    pub(crate) state: Option<i16>,
}

#[post("/api/events", format = "json", data = "<create_event_data>")]
pub fn create_event(
    settings: &State<Settings>,
    cookie: SessionCookie,
    create_event_data: Json<CreateEventData>,
) -> Result<Json<Event>, Json<ApiErrorWrapper>> {
    let caller = parse_member_cookie(cookie.member)?;
    if !caller.has_permission(crate::permissions::modules::event_management::events::CREATE.to_string()) {
        return Err(Json(
            ApiError::new(403, "Keine Berechtigung, neue Einsätze anzulegen!".to_string()).to_wrapper(),
        ));
    }

    let ecd = create_event_data.into_inner();

    let entity_id2 = match generate_entity(settings){
        Ok(ent) => ent,
        Err(_e) => return Err(Json(ApiError::new(500, "Konnte keine neue Entität anlegen!".to_string()).to_wrapper()))
    };

    let start = match NaiveDateTime::parse_from_str(&ecd.start, "%Y-%m-%dT%H:%M") {
        Ok(start) => start,
        Err(e) => {
            error!("Couldn't parse start datetime: {}", e);
            return Err(Json(ApiError::new(400, "Das eingegebene Datum konnte nicht verarbeitet werden.".to_string()).to_wrapper()))
        }
    };

    let end = match NaiveDateTime::parse_from_str(&ecd.end, "%Y-%m-%dT%H:%M") {
        Ok(start) => start,
        Err(e) => {
            error!("Couldn't parse end datetime: {}", e);
            return Err(Json(ApiError::new(400, "Das eingegebene Datum konnte nicht verarbeitet werden.".to_string()).to_wrapper()))
        }
    };

    if end <= start {
        return Err(Json(ApiError::new(400, "Die Endzeit darf nicht vor der Startzeit liegen.".to_string()).to_wrapper()))
    }

    let input: Event = Event{
        entity_id: entity_id2,
        name: ecd.name,
        start,
        end,
        site: ecd.site,
        organiser_id: parse_option_uuid(ecd.organiser_id)?,
        etype: parse_option_uuid(ecd.etype)?,
        contact_on_site_name: ecd.contact_on_site_name,
        contact_on_site_phone: ecd.contact_on_site_phone,
        member_responsible: parse_option_uuid(ecd.member_responsible)?,
        related_group: parse_option_uuid(ecd.related_group)?,
        other: ecd.other,
        other_intern: ecd.other_intern,
        related_request: None,
        state: 2
    };

    match add_event(settings, input){
        Ok(event) => Ok(Json(event)),
        Err(e) => Err(translate_diesel(e))
    }
}