use rocket::serde::json::Json;
use rocket::State;

use crate::database::controller::events::set_event_unit_position;
use crate::database::controller::events::templates::vehicle_positions::update_eu_vehicle_position;
use crate::database::model::events::{EventUnitPosition, EventUnitVehiclePosition};
use crate::helper::session_cookies::model::SessionCookie;
use crate::helper::settings::Settings;
use crate::helper::translate_diesel_error::translate_diesel;
use crate::modules::api::events::event_units::position::create::CreatePositionData;
use crate::modules::api::member_management::controller::parser::{parse_member_cookie, parse_uuid_string};
use crate::modules::api::model::api_outcome::{ApiError, ApiErrorWrapper};

#[put("/api/events/units/positions/<entity_id>", format = "json", data = "<update_position_data>")]
pub fn update_event_unit_position(
    settings: &State<Settings>,
    cookie: SessionCookie,
    update_position_data: Json<CreatePositionData>,
    entity_id: String,
) -> Result<Json<EventUnitPosition>, Json<ApiErrorWrapper>> {
    let caller = parse_member_cookie(cookie.member)?;
    if !caller.has_permission(crate::permissions::modules::event_management::events::EDIT.to_string()) {
        return Err(Json(
            ApiError::new(403, "Keine Berechtigung Einsatzpositionen anzulegen!".to_string()).to_wrapper(),
        ));
    }

    let update_position_data = update_position_data.into_inner();

    let requirements = match serde_json::from_str(&update_position_data.requirements){
        Ok(req) => req,
        Err(e) => {
            warn!("Couldn't parse requirements: {}", e);
            return Err(Json(ApiError::new(400, format!("Couldn't parse requirements: {}", e)).to_wrapper()))
        }
    };

    let event_position = EventUnitPosition{
        entity_id: parse_uuid_string(entity_id)?,
        name: update_position_data.name,
        description: update_position_data.description,
        requirements
    };

    match set_event_unit_position(settings, event_position) {
        Ok(eup) => Ok(Json(eup)),
        Err(e) => Err(translate_diesel(e))
    }
}

#[derive(Queryable, Clone, Deserialize, Serialize)]
pub struct UpdateVehiclePositionData {
    pub entity_id: String,
    pub name: String,
    pub description: Option<String>,
    pub required_vehicle_category: String,
    pub template_id: String,
}

/// Update vehicle_position
/// Parameters:
/// * vehicle_position_id: entity_id in eu_vehicle_positions
/// Method: PUT
/// Returns: EventUnitVehiclePosition or ApiError as JSON
#[put("/api/events/units/vehicle_positions/<vehicle_position_id>", format = "json", data = "<update_vehicle_position_data>")]
pub fn update_vehicle_position(
    settings: &State<Settings>,
    cookie: SessionCookie,
    vehicle_position_id: String,
    update_vehicle_position_data: Json<UpdateVehiclePositionData>,
) -> Result<Json<EventUnitVehiclePosition>, Json<ApiErrorWrapper>> {
    let caller = parse_member_cookie(cookie.member)?;
    if !caller.has_permission(crate::permissions::modules::event_management::events::EDIT.to_string()) {
        return Err(Json(
            ApiError::new(403, "Keine Berechtigung Fahrzeugposition zu verändern!".to_string()).to_wrapper(),
        ));
    }

    let data = update_vehicle_position_data.into_inner();

    if data.entity_id != vehicle_position_id {
        return Err(Json(
            ApiError::new(400, "vehicle_position_id in URI doesn't match PUT data!".to_string()).to_wrapper(),
        ));
    }


    let pos = EventUnitVehiclePosition {
        entity_id: parse_uuid_string(data.entity_id)?,
        name: data.name,
        description: data.description,
        required_vehicle_category: parse_uuid_string(data.required_vehicle_category)?,
        template_id: parse_uuid_string(data.template_id)?,
    };

    match update_eu_vehicle_position(settings, pos) {
        Ok(pos) => Ok(Json(pos)),
        Err(e) => Err(translate_diesel(e))
    }
}