use rocket::serde::json::Json;
use rocket::State;

use crate::database::controller::events::{EventUnitTemplatePosition, RawEventUnitTemplatePosition, update_eu_template, update_positions_templates};
use crate::database::model::events::EventUnitTemplate;
use crate::helper::session_cookies::model::SessionCookie;
use crate::helper::settings::Settings;
use crate::helper::translate_diesel_error::translate_diesel;
use crate::modules::api::events::event_units::templates::create::CreateTemplateData;
use crate::modules::api::member_management::controller::parser::{parse_member_cookie, parse_uuid_string};
use crate::modules::api::model::api_outcome::{ApiError, ApiErrorWrapper};

#[put("/api/events/units/templates/<template_id>", format = "json", data = "<template>")]
pub fn update_template(
    settings: &State<Settings>,
    cookie: SessionCookie,
    template_id: String,
    template: Json<CreateTemplateData>,
) -> Result<Json<EventUnitTemplate>, Json<ApiErrorWrapper>> {
    let caller = parse_member_cookie(cookie.member)?;
    if !caller.has_permission(crate::permissions::modules::event_management::events::EDIT.to_string()) {
        return Err(Json(
            ApiError::new(403, "Keine Berechtigung Vorlagen zu verändern!".to_string()).to_wrapper(),
        ));
    }

    let template = template.into_inner();

    let template = EventUnitTemplate{
        entity_id: parse_uuid_string(template_id)?,
        name: template.name,
        description: template.description
    };

    match update_eu_template(settings, template) {
        Ok(template) => Ok(Json(template)),
        Err(e) => Err(translate_diesel(e))
    }
}

#[derive(Queryable, Clone, Deserialize, Serialize)]
pub struct PositionTemplateData {
    pub position_template_id: String,
    pub template_id: String,
    pub position_entity_id: String,
    pub tag: Option<String>,
    pub num: String,
}

/// Updates eu_positions_templates
/// Parameters:
/// * template_id: current template as uuid
/// * position_id: current position as uuid
/// * position_template_id: position_template to edit
/// * position_template_data: JSON data for update
#[put("/api/events/units/templates/<template_id>/positions/<position_id>/<position_template_id>", format = "json", data = "<position_template_data>")]
pub fn update_position_for_template(
    settings: &State<Settings>,
    cookie: SessionCookie,
    position_template_data: Json<PositionTemplateData>,
    template_id: String,
    position_id: String,
    position_template_id: String,
) -> Result<Json<EventUnitTemplatePosition>, Json<ApiErrorWrapper>> {
    let caller = parse_member_cookie(cookie.member)?;
    if !caller.has_permission(crate::permissions::modules::event_management::events::EDIT.to_string()) {
        return Err(Json(
            ApiError::new(403, "Keine Berechtigung Vorlagen zu verändern!".to_string()).to_wrapper(),
        ));
    }

    let data = position_template_data.into_inner();

    if data.template_id != template_id || data.position_entity_id != position_id || data.position_template_id != position_template_id {
        return Err(Json(
            ApiError::new(400, "template_id or position_entity_id or position_template_id in URI doesn't match PUT data!".to_string()).to_wrapper(),
        ));
    }

    let num = match data.num.parse() {
        Ok(num) => num,
        Err(e) => {
            warn!("Wrong data type for num while putting position in template: {}", e);
            return Err(Json(ApiError::new(400, "Wrong data type for num.".to_string()).to_wrapper()))
        }
    };

    let res = RawEventUnitTemplatePosition {
        position_entity_id: parse_uuid_string(position_id)?,
        template_id: parse_uuid_string(template_id)?,
        position_template_id: parse_uuid_string(position_template_id)?,
        num,
        tag: data.tag,
    };

    match update_positions_templates(settings, res) {
        Ok(res) => Ok(Json(res)),
        Err(e) => Err(translate_diesel(e))
    }
}