use chrono::{Local, NaiveDateTime};
use rocket::serde::json::Json;
use rocket::State;

use crate::database::controller::entities::generate_entity;
use crate::database::controller::event_requests::add_event_request;
use crate::database::model::event_requests::EventRequest;
use crate::helper::session_cookies::model::SessionCookie;
use crate::helper::settings::Settings;
use crate::helper::translate_diesel_error::translate_diesel;
use crate::modules::api::member_management::controller::parser::{parse_member_cookie, parse_option_uuid};
use crate::modules::api::model::api_outcome::{ApiError, ApiErrorWrapper};

#[derive(Queryable, Clone, Deserialize, Serialize)]
pub struct RequestData {
    pub(crate) entity_id: Option<String>,
    pub(crate) name: String,
    pub(crate) timescale: Option<String>,
    pub(crate) site: Option<String>,
    pub(crate) organiser_id: Option<String>,
    pub(crate) etype: Option<String>,
    pub(crate) contact_on_site_name: Option<String>,
    pub(crate) contact_on_site_phone: Option<String>,
    pub(crate) related_group: Option<String>,
    pub(crate) specials: Option<String>,
    pub(crate) requested: Option<String>,
    pub(crate) expected_attendees: Option<String>,
    pub(crate) risk_potential: Option<String>,
    pub(crate) received: Option<String>,
    pub(crate) state: Option<String>,
}

#[post("/api/event_requests", format = "json", data = "<create_request_data>")]
pub fn create_event_request(
    settings: &State<Settings>,
    cookie: SessionCookie,
    create_request_data: Json<RequestData>,
) -> Result<Json<EventRequest>, Json<ApiErrorWrapper>> {
    let caller = parse_member_cookie(cookie.member)?;
    if !caller.has_permission(crate::permissions::modules::event_management::requests::CREATE.to_string()) {
        return Err(Json(
            ApiError::new(403, "Keine Berechtigung, neue Einsatzanfragen anzulegen!".to_string()).to_wrapper(),
        ));
    }
    
    let crd = create_request_data.into_inner();

    let entity_id = match generate_entity(settings){
        Ok(ent) => ent,
        Err(_e) => return Err(Json(ApiError::new(500, "Konnte keine neue Entität anlegen!".to_string()).to_wrapper()))
    };

    let risk_potential : Option<i32> = match crd.risk_potential{
        Some(rp) => match rp.parse() {
            Ok(rp) => Some(rp),
            Err(e) => {
                warn!("Couldn't parse risk potential for new request: {}", e);
                return Err(Json(ApiError::new(500, "couldn't parse risk potential".to_string()).to_wrapper()))
            }
        },
        None => None,
    };

    let expected_attendees : Option<i32> = match crd.expected_attendees{
        Some(ea) => match ea.parse() {
            Ok(ea) => Some(ea),
            Err(e) => {
                warn!("Couldn't parse expected attendees for new request: {}", e);
                return Err(Json(ApiError::new(500, "couldn't parse expected attendees".to_string()).to_wrapper()))
            }
        },
        None => None,
    };

    let received = match crd.received{
        Some(mut rec) => {
                    rec += "T00:00";
                    match NaiveDateTime::parse_from_str(&rec, "%Y-%m-%dT%H:%M") {
                        Ok(rec) => rec,
                        Err(e) => {
                            error!("Couldn't parse start datetime: {}", e);
                            return Err(Json(ApiError::new(400, "Das eingegebene Datum konnte nicht verarbeitet werden.".to_string()).to_wrapper()))
                        }
                    }
        },
        None => {
            Local::now().naive_local()
        }
    };

    let input = EventRequest{
        entity_id,
        name: crd.name,
        timescale: crd.timescale,
        site: crd.site,
        organiser_id: parse_option_uuid(crd.organiser_id)?,
        etype: parse_option_uuid(crd.etype)?,
        related_group: None,
        contact_on_site_name: crd.contact_on_site_name,
        contact_on_site_phone: crd.contact_on_site_phone,
        specials: crd.specials,
        requested: crd.requested,
        expected_attendees,
        risk_potential,
        state: 1,
        received,
    };

    match add_event_request(&settings, input){
        Ok(request) => Ok(Json(request)),
        Err(e) => Err(translate_diesel(e))
    }
}