use rocket::serde::json::Json;
use rocket::State;

use crate::database::controller::groups::update_group_core_data;
use crate::database::controller::members::check_access_to_resource;
use crate::database::controller::members_groups::{add_member_to_group, add_member_to_state};
use crate::helper::session_cookies::model::SessionCookie;
use crate::helper::settings::Settings;
use crate::helper::translate_diesel_error::translate_diesel;
use crate::modules::api::member_management::controller::parser::{parse_member_cookie, parse_uuid_string};
use crate::modules::api::model::api_outcome::{ApiError, ApiErrorWrapper};
use crate::modules::member_management::model::groups::GroupUpdateData;

#[put("/api/groups/<group_id>/members/<member_id>", format = "json")]
pub fn put_member_in_group(settings: &State<Settings>, cookie: SessionCookie, group_id: String, member_id: String) -> Result<(), Json<ApiErrorWrapper>> {
    let caller = parse_member_cookie(cookie.member)?;
    let member_id = parse_uuid_string(member_id)?;
    let group_id = parse_uuid_string(group_id)?;

    if !check_access_to_resource(settings, caller.entity_id, group_id, crate::permissions::modules::member_management::groups::members::EDIT) {
        return Err(Json(ApiError::new(403, "Keine Berechtigung Gruppenmitglieder zu ändern!".to_string()).to_wrapper()))
    }

    match add_member_to_group(settings, member_id, group_id){
        Ok(_) => Ok(()),
        Err(e) => Err(translate_diesel(e))
    }
}

#[put("/api/groups/<group_id>", format = "json", data = "<update_group_data>")]
pub fn update_group(settings: &State<Settings>, cookie: SessionCookie, update_group_data: Json<GroupUpdateData>, group_id: String) -> Result<(), Json<ApiErrorWrapper>>{
    let caller = parse_member_cookie(cookie.member)?;
    let group_id = parse_uuid_string(group_id)?;

    if !check_access_to_resource(settings, caller.entity_id, group_id, crate::permissions::modules::member_management::groups::core::EDIT){
        return Err(Json(ApiError::new(403, "Keine Berechtigung Gruppe zu ändern!".to_string()).to_wrapper()))
    }

    match update_group_core_data(settings, group_id, update_group_data.into_inner()){
        Ok(_) => Ok(()),
        Err(e) => Err(translate_diesel(e))
    }
}

/// Put member in state for group
/// Parameters:
/// * group_id: group_id as uuid in string
/// * member_id: members entity_id as uuid in string
/// * state_id: states id as uuid in string OR null if state should be set to NULL
#[put("/api/groups/<group_id>/members/<member_id>/state/<state_id>", format = "json")]
pub fn put_member_in_state(settings: &State<Settings>, cookie: SessionCookie, group_id: String, member_id: String, state_id: String) -> Result<(), Json<ApiErrorWrapper>>{
    let caller = parse_member_cookie(cookie.member)?;
    let member_id = parse_uuid_string(member_id)?;
    let group_id = parse_uuid_string(group_id)?;
    let state_id = if state_id == "null" {
        None
    } else {
        Some(parse_uuid_string(state_id)?)
    };

    if !check_access_to_resource(settings, caller.entity_id, group_id, crate::permissions::modules::member_management::groups::members::EDIT){
        return Err(Json(ApiError::new(403, "Keine Berechtigung Gruppenmitglieder zu ändern!".to_string()).to_wrapper()))
    }

    match add_member_to_state(settings, member_id, group_id, state_id){
        Ok(_) => Ok(()),
        Err(e) => Err(match e {
            diesel::result::Error::DatabaseError(kind, _) => match kind {
                diesel::result::DatabaseErrorKind::UniqueViolation => Json(
                    ApiError::new(
                        409,
                        "Dieses Mitglied hat diesen Status bereits in einer anderen Gruppe!".to_string(),
                    )
                        .to_wrapper(),
                ),
                _ => Json(
                    ApiError::new(500, "Es ist ein Datenbankfehler aufgetreten.".to_string())
                        .to_wrapper(),
                ),
            },
            diesel::result::Error::NotFound => Json(
                ApiError::new(
                    404,
                    "Die angeforderte Ressource konnte nicht gefunden werden.".to_string(),
                )
                    .to_wrapper(),
            ),
            _ => Json(
                ApiError::new(500, "Es ist ein Datenbankfehler aufgetreten.".to_string()).to_wrapper(),
            ),
        })
    }
}