use rocket::serde::json::Json;
use rocket::State;

use crate::database::controller::permissions::{add_permission_to_role, get_role_permission_id, remove_permission_from_role};
use crate::database::controller::roles::{add_member_to_role, add_permission_context, remove_member_from_role, remove_permission_context, update_role};
use crate::database::model::roles::Role;
use crate::helper::session_cookies::model::SessionCookie;
use crate::helper::settings::Settings;
use crate::helper::translate_diesel_error::translate_diesel;
use crate::modules::api::member_management::controller::parser::{parse_member_cookie, parse_uuid};
use crate::modules::api::model::api_outcome::{ApiError, ApiErrorWrapper};

/// Add permission to role
///
/// Returns new role_permission_id
///
/// # Api Call
/// * POST
/// * /api/roles/<role_id>/permissions/<permission_id>
///
/// # Api Result
/// * Result JSON<uuid::Uuid> / ApiErrorWrapper
///
/// # Required permissions
/// * modules.settings.role_permissions.edit
#[post("/api/roles/<role_id>/permissions/<permission_id>", format = "json")]
pub fn api_add_permission_to_role(
    settings: &State<Settings>,
    cookie: SessionCookie,
    role_id: String,
    permission_id: String,
) -> Result<Json<uuid::Uuid>, Json<ApiErrorWrapper>> {
    let caller = parse_member_cookie(cookie.member)?;

    if !caller.has_permission(crate::permissions::modules::settings::role_permissions::EDIT.to_string()){
        return Err(Json(ApiError::new(403, "No permission to edit role permissions".to_string()).to_wrapper()))
    }

    match add_permission_to_role(settings, &permission_id, &role_id) {
        Ok(res) => Ok(Json(res)),
        Err(e) => Err(translate_diesel(e))
    }
}


/// Remove permission to role
///
/// Returns nothing
///
/// # Api Call
/// * DELETE
/// * /api/roles/<role_id>/permissions/<permission_id>
///
/// # Api Result
/// * Result JSON<()> / ApiErrorWrapper
///
/// # Required permissions
/// * modules.settings.role_permissions.edit
#[delete("/api/roles/<role_id>/permissions/<permission_id>", format = "json")]
pub fn api_remove_permission_from_role(
    settings: &State<Settings>,
    cookie: SessionCookie,
    role_id: String,
    permission_id: String,
) -> Result<Json<()>, Json<ApiErrorWrapper>> {
    let caller = parse_member_cookie(cookie.member)?;

    if !caller.has_permission(crate::permissions::modules::settings::role_permissions::EDIT.to_string()){
        return Err(Json(ApiError::new(403, "No permission to edit role permissions".to_string()).to_wrapper()))
    }

    match remove_permission_from_role(settings, &permission_id, &role_id) {
        Ok(()) => Ok(Json(())),
        Err(e) => Err(translate_diesel(e))
    }
}

/// Add context to role & permission
///
/// Returns nothing
///
/// # Api Call
/// * POST
/// * /api/roles/<role_id>/permissions/<permission_id>/context/<context_id>
///
/// # Api Result
/// * Result JSON<()> / ApiErrorWrapper
///
/// # Required permissions
/// * modules.settings.role_permissions.edit
#[post("/api/roles/<role_id>/permissions/<permission_id>/context/<context_id>", format = "json")]
pub fn api_add_context(
    settings: &State<Settings>,
    cookie: SessionCookie,
    role_id: String,
    permission_id: String,
    context_id: String,
) -> Result<Json<()>, Json<ApiErrorWrapper>> {
    let caller = parse_member_cookie(cookie.member)?;

    if !caller.has_permission(crate::permissions::modules::settings::role_permissions::EDIT.to_string()){
        return Err(Json(ApiError::new(403, "No permission to edit role permissions".to_string()).to_wrapper()))
    }

    let rpi = match get_role_permission_id(settings, &permission_id, &role_id) {
        Ok(rpi) => match rpi {
            Some(rpi) => rpi,
            None => {
                warn!("Couldn't find role_permission_id");
                return Err(Json(ApiError::new(404, "role_permission_id not found.".to_string()).to_wrapper()))
            }
        },
        Err(e) => {
            warn!("Couldn't get role_permission_id: {}", e);
            return Err(Json(ApiError::new(500, "Couldn't get role_permission_id.".to_string()).to_wrapper()))
        },
    };

    let context_id = parse_uuid(&context_id)?;

    match add_permission_context(settings, rpi, context_id){
        Ok(()) => Ok(Json(())),
        Err(e) => {Err(translate_diesel(e))}
    }
}

/// Removes context from role & permission combo
///
/// Returns nothing
///
/// # Api Call
/// * POST
/// * /api/roles/<role_id>/permissions/<permission_id>/context/<context_id>
///
/// # Api Result
/// * Result JSON<()> / ApiErrorWrapper
///
/// # Required permissions
/// * modules.settings.role_permissions.edit
#[delete("/api/roles/<role_id>/permissions/<permission_id>/context/<context_id>", format = "json")]
pub fn api_remove_context(
    settings: &State<Settings>,
    cookie: SessionCookie,
    role_id: String,
    permission_id: String,
    context_id: String,
) -> Result<Json<()>, Json<ApiErrorWrapper>> {
    let caller = parse_member_cookie(cookie.member)?;

    if !caller.has_permission(crate::permissions::modules::settings::role_permissions::EDIT.to_string()){
        return Err(Json(ApiError::new(403, "No permission to edit role permissions".to_string()).to_wrapper()))
    }

    let rpi = match get_role_permission_id(settings, &permission_id, &role_id) {
        Ok(rpi) => match rpi {
            Some(rpi) => rpi,
            None => return Err(Json(ApiError::new(404, "role_permission_id not found.".to_string()).to_wrapper()))
        },
        Err(e) => {
            warn!("Couldn't get role_permission_id: {}", e);
            return Err(Json(ApiError::new(500, "Couldn't get role_permission_id.".to_string()).to_wrapper()))
        },
    };

    let context_id = parse_uuid(&context_id)?;

    match remove_permission_context(settings, rpi, context_id){
        Ok(()) => Ok(Json(())),
        Err(e) => {Err(translate_diesel(e))}
    }
}

/// Add member to role
///
/// Returns nothing
///
/// # Api Call
/// * POST
/// * /api/roles/<role_id>/members/<member_id>
///
/// # Api Result
/// * Result JSON<()> / ApiErrorWrapper
///
/// # Required permissions
/// * modules.settings.role_permissions.edit
#[post("/api/roles/<role_id>/members/<member_id>", format = "json")]
pub fn api_add_member_to_role(
    settings: &State<Settings>,
    cookie: SessionCookie,
    role_id: String,
    member_id: String,
) -> Result<Json<()>, Json<ApiErrorWrapper>> {
    let caller = parse_member_cookie(cookie.member)?;

    if !caller.has_permission(crate::permissions::modules::settings::role_permissions::EDIT.to_string()){
        return Err(Json(ApiError::new(403, "No permission to edit roles".to_string()).to_wrapper()))
    }

    let member_id = parse_uuid(&member_id)?;

    match add_member_to_role(settings, member_id, role_id) {
        Ok(()) => Ok(Json(())),
        Err(e) => Err(translate_diesel(e))
    }
}

/// Removes member from role
///
/// Returns nothing
///
/// # Api Call
/// * DELETE
/// * /api/roles/<role_id>/members/<member_id>
///
/// # Api Result
/// * Result JSON<()> / ApiErrorWrapper
///
/// # Required permissions
/// * modules.settings.role_permissions.edit
#[delete("/api/roles/<role_id>/members/<member_id>", format = "json")]
pub fn api_remove_member_from_role(
    settings: &State<Settings>,
    cookie: SessionCookie,
    role_id: String,
    member_id: String,
) -> Result<Json<()>, Json<ApiErrorWrapper>> {
    let caller = parse_member_cookie(cookie.member)?;

    if !caller.has_permission(crate::permissions::modules::settings::role_permissions::EDIT.to_string()){
        return Err(Json(ApiError::new(403, "No permission to edit roles".to_string()).to_wrapper()))
    }

    let member_id = parse_uuid(&member_id)?;

    match remove_member_from_role(settings, member_id, role_id) {
        Ok(()) => Ok(Json(())),
        Err(e) => Err(translate_diesel(e))
    }
}

/// Updates role data
///
/// Returns Role
///
/// # Api Call
/// * PUT
/// * /api/roles/<role_id>
///
/// # Api Result
/// * Result JSON<[Role]> / ApiErrorWrapper
///
/// # Required permissions
/// * modules.settings.role_permissions.edit
#[put("/api/roles/<role_id>", format = "json", data = "<role_data>")]
pub fn api_update_role(
    settings: &State<Settings>,
    cookie: SessionCookie,
    role_id: String,
    role_data: Json<Role>,
) -> Result<Json<Role>, Json<ApiErrorWrapper>> {
    let role_data = role_data.into_inner();
    let caller = parse_member_cookie(cookie.member)?;

    if !caller.has_permission(crate::permissions::modules::settings::role_permissions::EDIT.to_string()){
        return Err(Json(ApiError::new(403, "No permission to edit roles".to_string()).to_wrapper()))
    }

    match update_role(settings, role_id, role_data) {
        Ok(res) => Ok(Json(res)),
        Err(e) => Err(translate_diesel(e))
    }
}