use bigdecimal::BigDecimal;
use rocket::serde::json::Json;
use rocket::State;

use crate::database::controller::billing::personnel_billing::RawPersonnelBilling;
use crate::database::controller::billing::personnel_billing_rates::{get_billing_rate, get_billing_rates};
use crate::database::controller::events::{get_position_instance, get_position_instances};
use crate::helper::session_cookies::model::SessionCookie;
use crate::helper::translate_diesel_error::translate_diesel;
use crate::modules::api::member_management::controller::parser::{parse_member_cookie, parse_uuid_string};
use crate::modules::api::model::api_outcome::{ApiError, ApiErrorWrapper};
use crate::modules::api::personnel_billing::calculation::calculate_hours;
use crate::Settings;

#[derive(Queryable, Clone, Deserialize, Serialize)]
pub struct AddPersonnelBillingEntryData {
    pub(crate) position_instance_id: uuid::Uuid,
    pub(crate) billing_rate_id: Option<uuid::Uuid>,
    pub(crate) money_from_lump_sum: Option<String>,
    pub(crate) money_for_time: Option<String>,
}

#[post(
"/api/personnel_billing",
format = "json", data = "<data>"
)]
pub fn add_personnel_billing_entry(
    settings: &State<Settings>,
    cookie: SessionCookie,
    data: Json<AddPersonnelBillingEntryData>,
) -> Result<(), Json<ApiErrorWrapper>> { //TODO: return entry
    let caller = parse_member_cookie(cookie.member.clone())?;
    //TODO: allow member_responsible for event to bypass this permission
    if !caller
        .has_permission(crate::permissions::modules::event_billing::personnel::EDIT.to_string())
    {
        return Err(Json(
            ApiError::new(
                403,
                "Keine Berechtigung Einsatzabrechnung zu bearbeiten!".to_string(),
            )
                .to_wrapper(),
        ));
    }

    let data = data.into_inner();

    if (data.money_from_lump_sum.is_none() || data.money_for_time.is_none()) && data.billing_rate_id.is_none() {
        return Err(Json(
            ApiError::new(
                400,
                "Missing either money_from_lump_sum+money_for_time or billing_rate_id.".to_string(),
            )
                .to_wrapper(),
        ));
    }

    let position_instance = match get_position_instance(settings, data.position_instance_id) {
        Ok(res) => res,
        Err(e) => return Err(translate_diesel(e))
    };

    let fulfilled_time = if let Some(start) = position_instance.real_start_time {
        if let Some(end) = position_instance.real_end_time {
            calculate_hours(start, end)
        } else {
            None
        }
    } else {
        None
    };

    let fulfilled_time = match fulfilled_time {
        Some(ft) => ft,
        None => {
            error!("Couldn't calculate fulfilled time for position_instance {}!", position_instance.position_instance_id);
            return Err(Json(
                ApiError::new(
                    500,
                    "Couldn't calculate fulfilled time.".to_string(),
                )
                    .to_wrapper(),
            ));
        }
    };

    let money_for_time: BigDecimal;
    let money_from_lump_sum: BigDecimal;

    match data.billing_rate_id {
        Some(bri) => {
            let billing_rate = match get_billing_rate(settings, bri) {
                Ok(res) => res,
                Err(e) => return Err(translate_diesel(e))
            };
            money_for_time = billing_rate.payment_per_hour.clone() * BigDecimal::from(fulfilled_time);
            money_from_lump_sum = billing_rate.lump_sum.clone();
        }
        None => {
            match data.money_for_time {
                Some(mft) => {
                    match data.money_from_lump_sum {
                        Some(mfls) => {
                            money_for_time = BigDecimal::from(mft.parse::<f32>().unwrap()); //TODO: do not unwrap
                            money_from_lump_sum = BigDecimal::from(mfls.parse::<f32>().unwrap()); //TODO: do not unwrap
                        }
                        None => return Err(Json(
                            ApiError::new(
                                400,
                                "Can't create personnel billing entry for positon_instance with billable = false!".to_string(),
                            )
                                .to_wrapper(),
                        ))
                    }
                }
                None => return Err(Json(
                    ApiError::new(
                        400,
                        "Can't create personnel billing entry for positon_instance with billable = false!".to_string(),
                    )
                        .to_wrapper(),
                ))
            }
        }
    }

    let total_money = money_from_lump_sum.clone() + money_for_time.clone();

    let new_entry = RawPersonnelBilling {
        position_instance_id: data.position_instance_id,
        member_id: position_instance.taken_by.unwrap(),
        fulfilled_time,
        money_for_time,
        money_from_lump_sum,
        total_money,
    };
    match crate::database::controller::billing::personnel_billing::create(settings, new_entry) {
        Ok(_) => Ok(()),
        Err(e) => Err(translate_diesel(e))
    }
}