use rocket::http::Status;
use rocket::State;
use rocket_dyn_templates::Template;

use crate::database::controller::events::get_event;
use crate::database::controller::members::check_access_to_resource;
use crate::database::model::events::Event;
use crate::helper::session_cookies::model::SessionCookie;
use crate::helper::sitebuilder::model::general::{Footer, Header, Script, Stylesheet};
use crate::helper::sitebuilder::model::sidebar::Sidebar;
use crate::modules::api::member_management::controller::parser::parse_uuid_string;
use crate::modules::event_billing::event::EventBilling;
use crate::Settings;

/// UI for viewing, editing & approving event unit times.
/// required permissions:
///     * modules.event_billing.start_end_times.view OR be member_responsible for event to view real start/end times
///     * modules.event_billing.start_end_times.edit OR be member_responsible for event to edit real start/end times
#[get("/portal/eb/approve?<event_id>&<stage>")]
pub fn approve(cookie: SessionCookie, settings: &State<Settings>, event_id: String, stage: String) -> Result<Template, Status> {
    let caller = match cookie.member {
        //Unwraps member from cookie or send user to login if no member specified (user skipped member selection)
        Some(caller) => caller,
        None => return Err(Status::Unauthorized),
    };

    match stage.parse() {
        Ok(stage) => {
            if !check_access_to_resource(settings, caller.entity_id, stage, crate::permissions::modules::event_billing::APPROVE) {
                return Err(Status::Forbidden);
            }
        },
        Err(e) => {
            warn!("Unparsable stage in approve: {}", e);
            return Err(Status::BadRequest);
        },
    }

    let event_id: uuid::Uuid = match event_id.parse() {
        Ok(event_id) => event_id,
        Err(e) => {
            warn!("Unparsable event_id in approve: {}", e);
            return Err(Status::BadRequest);
        }
    };

    let event = match get_event(settings, event_id) {
        Ok(event) => event,
        Err(e) => {
            warn!("Couldn't load event: {}", e);
            return Err(Status::NotFound);
        }
    };

    //Do not allow editing times before event is closed
    if event.state < 7 {
        error!("User tried to approve before event was billed");
        return Err(Status::BadRequest);
    }

    let header = Header {
        html_language: "de".to_string(),
        site_title: "Einsatzabrechnung".to_string(),
        stylesheets: vec![Stylesheet {
            path: "/css/errms.css".to_string(),
        }],
    };
    let footer = Footer {
        scripts: vec![Script {
            path: "/js/event_billing.js".to_string(),
        }, Script {
            path: "/js/mini_searchbar.js".to_string(),
        }],
    };
    let mut sidebar = Sidebar::new(caller.clone());
    sidebar.event_billing.active = true;

    Ok(Template::render("module_eb_approve", EventBilling {
        header,
        footer,
        sidebar,
        caller: caller.entity_id,
        event,
    }))
}