use rocket::http::Status;
use rocket::State;
use rocket_dyn_templates::Template;

use crate::database::controller::events::get_event;
use crate::database::model::events::Event;
use crate::helper::session_cookies::model::SessionCookie;
use crate::helper::sitebuilder::model::general::{Footer, Header, Script, Stylesheet};
use crate::helper::sitebuilder::model::sidebar::Sidebar;
use crate::modules::event_billing::event::EventBilling;
use crate::Settings;

/// UI for viewing, editing & approving personnel billing for an event.
/// required permissions:
///     * modules.event_billing.personnel.view OR be member_responsible for event to view personnel billing details
///     * modules.event_billing.personnel.edit OR be member_responsible for event to edit personnel billing details
#[get("/portal/eb/personnel_billing?<event_id>")]
pub fn edit_personnel_billing(cookie: SessionCookie, settings: &State<Settings>, event_id: String) -> Result<Template, Status> {
    let caller = match cookie.member {
        //Unwraps member from cookie or send user to login if no member specified (user skipped member selection)
        Some(caller) => caller,
        None => return Err(Status::Unauthorized),
    };

    let event_id: uuid::Uuid = match event_id.parse() {
        Ok(event_id) => event_id,
        Err(e) => {
            warn!("Unparsable event_id in personnel_billing: {}", e);
            return Err(Status::BadRequest);
        }
    };

    let event = match get_event(settings, event_id) {
        Ok(event) => event,
        Err(e) => {
            warn!("Couldn't load event: {}", e);
            return Err(Status::NotFound);
        }
    };

    //Do not allow editing times before times are submitted
    if event.state < 6 {
        error!("User tried to edit personnel billing before times were submitted");
        return Err(Status::BadRequest);
    }
    //TODO: restrict "going-back" to members with specific permission (https://git.anghenfil.de/EinsatzOnline/EinsatzOnline/issues/21)

    if !caller.has_permission(crate::permissions::modules::event_billing::personnel::VIEW.to_string()) {
        match event.member_responsible {
            Some(resp) => {
                if caller.entity_id != resp {
                    return Err(Status::Forbidden);
                }
            }
            None => {
                return Err(Status::Forbidden);
            }
        }
    }

    let header = Header {
        html_language: "de".to_string(),
        site_title: "Einsatzabrechnung".to_string(),
        stylesheets: vec![Stylesheet {
            path: "/css/errms.css".to_string(),
        }],
    };
    let footer = Footer {
        scripts: vec![Script {
            path: "/js/event_billing.js".to_string(),
        }, Script {
            path: "/js/search2.js".to_string(),
        }],
    };
    let mut sidebar = Sidebar::new(caller.clone());
    sidebar.event_billing.active = true;

    Ok(Template::render("module_eb_personnel_billing", EventBilling {
        header,
        footer,
        sidebar,
        caller: caller.entity_id,
        event,
    }))
}